package de.adorsys.mobile.flutter_udf_plugin

import androidx.annotation.NonNull
import de.adorsys.android.unifieddatafingerprintlibrary.AddressType
import de.adorsys.android.unifieddatafingerprintlibrary.UDF
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.nio.charset.StandardCharsets


/** FlutterUdfPlugin */
class FlutterUdfPlugin : FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "flutter_udf_plugin")
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "getPlatformVersion" -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            "getHashForPhoneNumber" -> {
                val phoneNumber = call.argument<String>("phoneNumber")
                if (!phoneNumber.isNullOrEmpty()) {
                    val udfStringRepresentation = getUdfForPhoneNumber(phoneNumber)
                    result.success(udfStringRepresentation)
                } else {
                    result.error("400", "Missing argument phoneNumber", null)
                }
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    private fun getUdfForPhoneNumber(phoneNumber: String): String {
        val data = phoneNumber.toByteArray(StandardCharsets.UTF_8)
        val udf = UDF.dataToUDFBinary(data, AddressType.phone.name, 0, null, null)
        return udf.presentationBase32(125)
    }
}
